'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();





var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var ClickOutHandler = function (_Component) {
  inherits(ClickOutHandler, _Component);

  function ClickOutHandler() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, ClickOutHandler);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = ClickOutHandler.__proto__ || Object.getPrototypeOf(ClickOutHandler)).call.apply(_ref, [this].concat(args))), _this), _this.setRef = function (el) {
      _this.wrapper = el;
    }, _this.handleClickOut = function (ev) {
      if (_this.shouldFire(ev)) _this.props.onClickOut(ev);
    }, _this.events = ['mousedown', 'touchstart'], _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(ClickOutHandler, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      this.events.forEach(function (event) {
        document.addEventListener(event, _this2.handleClickOut);
      });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      var _this3 = this;

      this.events.forEach(function (event) {
        document.removeEventListener(event, _this3.handleClickOut);
      });
    }
  }, {
    key: 'getWrapper',
    value: function getWrapper() {
      return React__default.createElement(this.props.wrapWith || 'div', defineProperty({}, this.props.refProp, this.setRef), this.props.children);
    }
  }, {
    key: 'shouldFire',
    value: function shouldFire(ev) {
      return this.props.enabled && this.wrapper && !this.wrapper.contains(ev.target) && !this.props.ignoredElements.some(function (element) {
        return element && element.contains(ev.target);
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          children = _props.children,
          refProp = _props.refProp,
          wrapWith = _props.wrapWith;


      return Array.isArray(this.props.children) || wrapWith ? this.getWrapper() : React__default.cloneElement(React__default.Children.only(children), defineProperty({}, refProp, this.setRef));
    }
  }]);
  return ClickOutHandler;
}(React.Component);

ClickOutHandler.propTypes = {
  children: PropTypes.oneOfType([PropTypes.array, PropTypes.node]).isRequired,
  enabled: PropTypes.bool,
  ignoredElements: PropTypes.arrayOf(PropTypes.object),
  refProp: PropTypes.string,
  wrapWith: PropTypes.oneOfType([PropTypes.element, PropTypes.func, PropTypes.string]),
  onClickOut: PropTypes.func.isRequired
};
ClickOutHandler.defaultProps = {
  enabled: true,
  ignoredElements: [],
  refProp: 'ref',
  wrapWith: null
};

module.exports = ClickOutHandler;
